##############################################################################
# 
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
# 
# Copyright (c) Digital Creations.  All rights reserved.
# 
# This license has been certified as Open Source(tm).
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
# 
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
# 
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
# 
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
# 
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
# 
# 
# Disclaimer
# 
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
# 
# 
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
# 
##############################################################################

__doc__="""This is NotMail, a web application.

This module contains the NotMail class definition, as well as external
methods for Zope to create and install NotMail objects into the
database.

NotMail is a thin client, in email jargon it is a Mail User Agent.
NotMail uses smtp to send mail and imap to 'recieve' it.  NotMail does
not handle any detail of mail transport, composed messages get sent
straight to an smtp server which then deals with it, and to recieve
messages NotMail uses an imap server, in reality, NotMail could be
considered an 'imap browser'.

While in any one of the numerous smtplib or imaplib calls, **NotMail
will block your entire Zope process**.  If you are using this for
production (which you shouldn't cuz it's just version 0.0.7) make sure
your servers are snappy and your loads are light.  As soon as Zope
supports concurency (which should be Real Soon Now, ie 2.0) this
problem will go away.

Version Changes

0.0.1 - Genesis 0.0.2 - Got most forms working, transposed all work
from initial Bobo prototypes.  based on Sample Item application posted
to the bobo list by Digicool.

0.0.3 - NotMail is now a Folder object, fixed some bugs, cleaned out
some spurious arguments, made the dtml interface customizable. Added
lots of documentation.

0.0.4 - Figured out some stuff and added imap support, NotMail is now
a thin imap client.  What this means is that you must have an imap
server running for NotMail to interface.  imap passwords are gotten
from the user object, so please make sure that the Zope passwd and
imap passwd match.  IMAP is used in a sessionless way, much like
POP3.

0.0.5 - imap can return rfc822 formatted messages, which can then be
piped into the python rfc822 module.  Adding all this required some
dtml changes also.  What this means is that it now looks pretty.
Apears to hang when pointed toward an Exchange server, it worked
before so I broke it somehow.

0.0.6 - works with Exchange but buggy.  Moving and deleting mail
between boxes now works.

0.0.7 - Cleaned up the DTML (alot) and stopped passing
AUTHENTICATED_USER into methods, now snarfed out of REQUEST (duh).
Added sane defaults to notMailAdd.  Search interface is now cool but
it's easy to break (hit the shit key and a bunch of numbers, it will
barf) and it's a simplification of what imaps built in search language
can do.  Box names with spaces are url_quoted.

"""

import string
import smtplib

from Globals import HTMLFile, MessageDialog, package_home
import OFS.Folder, Persistence, Acquisition
import AccessControl.Role

from Products.IMAPAdapter import IMAPAdapter

addNotMailForm=HTMLFile('notMailAdd',globals())

def addNotMail(self, id, title='', default_domain='',
	       smtp_server='', smtp_port=25, imap_server='',
	       imap_port=143, path='', REQUEST=None):

    """
    Function is called by Zope to create and install a NotMail
    object.

    """
    n = NotMail(id, title, default_domain, smtp_server, smtp_port,
		imap_server, imap_port, path)
    self._setObject(id, n)
    # return to the parent object's manage_main
    if REQUEST is not None: return self.manage_main(self,REQUEST)



class NotMail(OFS.Folder.Folder,
	      Persistence.Persistent,
	      Acquisition.Implicit,
	      AccessControl.Role.RoleManager,
	      ):
    """NotMail v 0.0.6

    Hacks are clearly marked.

    """

    meta_type='NotMail'
    # XXX must get a new icon, any takers?
    icon='misc_/NotMail/item'

    manage_options=(
	{'label':'Contents',   'action':'manage_main'},
	{'label':'Properties', 'action':'manage_propertiesForm'},
	{'label':'View',       'action':''},
	{'label':'Security',   'action':'manage_access'},
	)
 
    __ac_permissions__=(
	('View management screens', ('manage_tabs','manage_main')),
	('Change permissions', ('manage_access',)),
	('Change NotMail Settings', ('manage_edit',)),
	('NotMail User',   ('',)),
	)
	

    def __init__(self, id, title='', default_domain, smtp_server,
                 smtp_port, imap_server, imap_port, RQUEST=None):
	""" The NotMail Object constructor

	Initializes the NotMail object.  Sets up some data
	and adds all the default documents into itself.
	"""

	self.debug=1

	self.id=id
	# here down should be done by manage_edit
	self.title=title

        self.__roles__=['NotMail']

        roles="NotMail, IMAPUser"
        IMAPAdapter.addIMAPAdapter(self.__of__(self), title, default_domain, smtp_server, 
                                   smtp_port, imap_server, imap_port,
                                   roles)

	# These documents are loaded into newly created 
	# NotMail objects. good place to customize your 
	# whole program layout if you dont' like mine.

	self.defaultDocs = [
	    ('index', 'NotMail'),
	    ('composer', 'Compose a Message'),
	    ('contact', 'Contact Developers'),
	    ('folder', 'Mailbox'),
	    ('message', 'Message'),
	    ('help', 'Help'),
	    ('noto', 'Message must have at least one recipient.'),
	    ('logout', 'You have been logged out'),
	    ('ZPL', 'The Zope Public License'),
	    ('imap_menu', 'Folder View'),
	    ('sentmail', 'Mail was sent')]

	# XXX create default documents here

	for id,title in self.defaultDocs:
	    try:
		self.defaultDocFile(id, title, id)
	    except:
		pass


    def index_html(self, REQUEST=None):
	""" The default method """
	return self.index(self, REQUEST)

    manage_main=HTMLFile('manage_main', globals())


    def manage_edit(self, title, default_domain, smtp_server,
		    smtp_port, imap_server, imap_port, path, REQUEST=None):
	""" Edit the NotMail Object
	
	"""
	self.title=title
	self.default_domain=default_domain
	self.smtp_server = smtp_server
	self.smtp_port = smtp_port
	self.imap_server = imap_server
	self.imap_port = imap_port
	self.path = path
	return self.manage_main(self, REQUEST)


    def composer_send(self, user, to=None, subject='', cc=None,
		      outgoing=None, body='', attachment=None, REQUEST=None):
	"""composer_send: Handles the 'Send' button
		
	"""
	if to == '':
	    """ can't do that! """
	    return HTMLFile("noto.dtml", globals())

	# insert some headers

	headers = "From: %s\nSubject: %s\n\n" % (user, subject)

	# split up all the recievers into a list
	# this can bomb the sendmail call if the list is malformed
	# need error checking

	to = ( \
		string.splitfields(string.strip(to), ',') \
		+ string.splitfields(string.strip(cc), ',') )

	body = headers + body

	# here we need to append any attachments
	# to the body after converting them to something
	# trasportable over mail, like MIME
	# hack this out better

	server = smtplib.SMTP(self.smtp_server, self.smtp_port)
	server.sendmail(user, to, body)
	server.quit()

	return self.index(self, REQUEST)


    def composer_cancel(self, REQUEST):
	""" composer_cancel: handles the 'Cancel' button

	Returns a fresh composer_form when the user clicks
	cancel.
	"""
	# fresh page
	return self.composer(self, REQUEST)


    def reply(self, box, msg, reply_all=None, REQUEST=None):
	""" reply to a message, quote the original """
        user = REQUEST['AUTHENTICATED_USER']
	omessage = user.imapGetUID(box, msg, REQUEST)

	isfrom = omessage.getaddr('From')
	if isfrom is None:
	    # hmm. if there's no from how did it get here to reply to
	    # it?
	    REQUEST['reply_to'] = ''
	else:
	    REQUEST['reply_to'] = isfrom[1]

	subject = omessage.getheader('Subject')
	if subject is None:
	    REQUEST['reply_subject'] = ''
	else:
	    REQUEST['reply_subject'] = string.strip(subject)

	if reply_all is not None:
	    cc = omessage.getaddr('cc')
	    if cc is None:
		REQUEST['reply_cc'] = ''
	    else:
		REQUEST['reply_to'] = cc

	body = string.split(omessage.fp.read(), '\n')

	REQUEST['reply_body'] = body
	return self.composer(self, REQUEST)

    def defaultDocFile(self, id, title, file):
	""" Adds a Document object read from disk

	Reads a dtml file from the NotMail directory and 
	insterts it into a fresh Document object.  Used
	during the initialization phase of new NotMail	
	objects to load in default information.
	Contributed by Amos Latteier.
	"""
	f=open('%s/Products/NotMail/%s.dtml' 
	       % (SOFTWARE_HOME, file))
	# Jim suggested below, but it doesn't work I need to RTFS I guess...
	# d = package_home(globals())
	# f=open(d + '%s.dtml' % file)
	file=f.read()
	f.close()
	self.manage_addDocument(id, title, file)


    def refresh_docs(self, REQUEST):
	""" refresh documents from disk (for debug purposes) """

	for id,title in self.defaultDocs:
	    try:
		self._delObject(id)
	    except:
		pass
	    try:
		self.defaultDocFile(id, title, id)
	    except:
		pass

	return self.manage_main(self, REQUEST)
